#include <iostream>
#include <unistd.h>
#include <map>
#include <fcntl.h>
#include <sys/stat.h>
#include <cstdlib>
#include <ctime>

using namespace std;

#include "queue.h"
#include "misc.h"
#include "mode.h"

typedef void (*proc)(const string &, const string &, const vector<string> &a);
					 
static map<string,proc> procs;

static void InitHooks()
{
	#define add(x) \
		extern void x(const string&, const string&, const vector<string> &a);\
		procs[#x] = x
	
	add(PRIVMSG);
	add(MODE);
	add(JOIN);
	add(PART);
	add(KICK);
	add(QUIT);
	
	#undef add
}
static void Hook(const string &from, const string &what, const string &to,
				 const vector<string> &words)
{
	map<string,proc>::iterator i;
	for(i=procs.begin(); i!=procs.end(); i++)
		if(i->first==what)
			i->second(from,to,words);
}

extern bool Idle();

// 10 ticks = one second
void LoopRead(long maxticks)
{
	while(!cin.bad())
	{
		vector<string>words;
		string s;
		
		for(;;)
		{
			cin.clear();
			cin.peek();
			if(!cin.eof())break;
			dequeue();
			if(Idle())return;
			usleep(100000);
			if(!maxticks--)return;
		}
		
		getline(cin, s);
		
		if(s[0] != ':')continue;
		
		unsigned a, b, m=s.size();
		
		while(s[m-1]==' ')m--;
	
		for(a=b=1; a<m; a++)
		{
			if(s[a]==' ')
			{
				words.insert(words.end(), s.substr(b, a-b));
				b = a+1;
				if(s[b]==':')
				{
					s.erase(0, b+1);
					words.insert(words.end(), s);
					break;
				}
			}
		}
		
		if(words.size() < 3)continue;
		
		string from=words[0], what=words[1], to=words[2];
		
		words.erase(words.begin());
		words.erase(words.begin());
		words.erase(words.begin());
		
		Hook(from, what, to, words);
	}
}

int main(int argc, const char **argv)
{
	srand(time(NULL));
	
	InitHooks();
	
	// stdin
	fcntl(0,F_SETFL,fcntl(0,F_GETFL)|O_NONBLOCK);
	
	bool havenick = false;
	while(--argc)
	{
		const char *s = *++argv;
		if(*s=='-')continue;
		if(!havenick)
		{
			mynick = s;
			cout << "NICK " << mynick << endl << flush;
			havenick=true;
		}
		botit.insert(botit.end(), s);
	}
	if(!havenick)
	{
		cerr <<
			"Who am I?\n"
			"Usage: script <mynick> [<friendbot> [...]]\n\n";
		return -1;
	}
	
	cout << "JOIN #gurb,#pkmn\n" << flush;
	/*
	if(mynick=="Dah")
		cout << "JOIN #hypnosis\n" << flush;
	else
		cout << "JOIN #hypnosis\n" << flush;
	
	cout << "JOIN #kerava\n" << flush;
	*/
	
	LoopRead(-1);
	
	return 0;
}
